import datetime
from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError
from ..models.lab_automation_benchling_app_errors import LabAutomationBenchlingAppErrors
from ..models.lab_automation_transform_status import LabAutomationTransformStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="LabAutomationTransform")


@attr.s(auto_attribs=True, repr=False)
class LabAutomationTransform:
    """  """

    _api_url: Union[Unset, str] = UNSET
    _blob_id: Union[Unset, None, str] = UNSET
    _custom_transform_id: Union[Unset, None, str] = UNSET
    _errors: Union[Unset, LabAutomationBenchlingAppErrors] = UNSET
    _id: Union[Unset, str] = UNSET
    _modified_at: Union[Unset, datetime.datetime] = UNSET
    _output_processor_id: Union[Unset, str] = UNSET
    _status: Union[Unset, LabAutomationTransformStatus] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("api_url={}".format(repr(self._api_url)))
        fields.append("blob_id={}".format(repr(self._blob_id)))
        fields.append("custom_transform_id={}".format(repr(self._custom_transform_id)))
        fields.append("errors={}".format(repr(self._errors)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("modified_at={}".format(repr(self._modified_at)))
        fields.append("output_processor_id={}".format(repr(self._output_processor_id)))
        fields.append("status={}".format(repr(self._status)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "LabAutomationTransform({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        api_url = self._api_url
        blob_id = self._blob_id
        custom_transform_id = self._custom_transform_id
        errors: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._errors, Unset):
            errors = self._errors.to_dict()

        id = self._id
        modified_at: Union[Unset, str] = UNSET
        if not isinstance(self._modified_at, Unset):
            modified_at = self._modified_at.isoformat()

        output_processor_id = self._output_processor_id
        status: Union[Unset, int] = UNSET
        if not isinstance(self._status, Unset):
            status = self._status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if api_url is not UNSET:
            field_dict["apiURL"] = api_url
        if blob_id is not UNSET:
            field_dict["blobId"] = blob_id
        if custom_transform_id is not UNSET:
            field_dict["customTransformId"] = custom_transform_id
        if errors is not UNSET:
            field_dict["errors"] = errors
        if id is not UNSET:
            field_dict["id"] = id
        if modified_at is not UNSET:
            field_dict["modifiedAt"] = modified_at
        if output_processor_id is not UNSET:
            field_dict["outputProcessorId"] = output_processor_id
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_api_url() -> Union[Unset, str]:
            api_url = d.pop("apiURL")
            return api_url

        try:
            api_url = get_api_url()
        except KeyError:
            if strict:
                raise
            api_url = cast(Union[Unset, str], UNSET)

        def get_blob_id() -> Union[Unset, None, str]:
            blob_id = d.pop("blobId")
            return blob_id

        try:
            blob_id = get_blob_id()
        except KeyError:
            if strict:
                raise
            blob_id = cast(Union[Unset, None, str], UNSET)

        def get_custom_transform_id() -> Union[Unset, None, str]:
            custom_transform_id = d.pop("customTransformId")
            return custom_transform_id

        try:
            custom_transform_id = get_custom_transform_id()
        except KeyError:
            if strict:
                raise
            custom_transform_id = cast(Union[Unset, None, str], UNSET)

        def get_errors() -> Union[Unset, LabAutomationBenchlingAppErrors]:
            errors: Union[Unset, Union[Unset, LabAutomationBenchlingAppErrors]] = UNSET
            _errors = d.pop("errors")

            if not isinstance(_errors, Unset):
                errors = LabAutomationBenchlingAppErrors.from_dict(_errors)

            return errors

        try:
            errors = get_errors()
        except KeyError:
            if strict:
                raise
            errors = cast(Union[Unset, LabAutomationBenchlingAppErrors], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_modified_at() -> Union[Unset, datetime.datetime]:
            modified_at: Union[Unset, datetime.datetime] = UNSET
            _modified_at = d.pop("modifiedAt")
            if _modified_at is not None and not isinstance(_modified_at, Unset):
                modified_at = isoparse(cast(str, _modified_at))

            return modified_at

        try:
            modified_at = get_modified_at()
        except KeyError:
            if strict:
                raise
            modified_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_output_processor_id() -> Union[Unset, str]:
            output_processor_id = d.pop("outputProcessorId")
            return output_processor_id

        try:
            output_processor_id = get_output_processor_id()
        except KeyError:
            if strict:
                raise
            output_processor_id = cast(Union[Unset, str], UNSET)

        def get_status() -> Union[Unset, LabAutomationTransformStatus]:
            status = UNSET
            _status = d.pop("status")
            if _status is not None and _status is not UNSET:
                try:
                    status = LabAutomationTransformStatus(_status)
                except ValueError:
                    status = LabAutomationTransformStatus.of_unknown(_status)

            return status

        try:
            status = get_status()
        except KeyError:
            if strict:
                raise
            status = cast(Union[Unset, LabAutomationTransformStatus], UNSET)

        lab_automation_transform = cls(
            api_url=api_url,
            blob_id=blob_id,
            custom_transform_id=custom_transform_id,
            errors=errors,
            id=id,
            modified_at=modified_at,
            output_processor_id=output_processor_id,
            status=status,
        )

        lab_automation_transform.additional_properties = d
        return lab_automation_transform

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def api_url(self) -> str:
        """ The canonical url of the transform in the API. """
        if isinstance(self._api_url, Unset):
            raise NotPresentError(self, "api_url")
        return self._api_url

    @api_url.setter
    def api_url(self, value: str) -> None:
        self._api_url = value

    @api_url.deleter
    def api_url(self) -> None:
        self._api_url = UNSET

    @property
    def blob_id(self) -> Optional[str]:
        if isinstance(self._blob_id, Unset):
            raise NotPresentError(self, "blob_id")
        return self._blob_id

    @blob_id.setter
    def blob_id(self, value: Optional[str]) -> None:
        self._blob_id = value

    @blob_id.deleter
    def blob_id(self) -> None:
        self._blob_id = UNSET

    @property
    def custom_transform_id(self) -> Optional[str]:
        if isinstance(self._custom_transform_id, Unset):
            raise NotPresentError(self, "custom_transform_id")
        return self._custom_transform_id

    @custom_transform_id.setter
    def custom_transform_id(self, value: Optional[str]) -> None:
        self._custom_transform_id = value

    @custom_transform_id.deleter
    def custom_transform_id(self) -> None:
        self._custom_transform_id = UNSET

    @property
    def errors(self) -> LabAutomationBenchlingAppErrors:
        if isinstance(self._errors, Unset):
            raise NotPresentError(self, "errors")
        return self._errors

    @errors.setter
    def errors(self, value: LabAutomationBenchlingAppErrors) -> None:
        self._errors = value

    @errors.deleter
    def errors(self) -> None:
        self._errors = UNSET

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def modified_at(self) -> datetime.datetime:
        """ DateTime the transform was last modified. """
        if isinstance(self._modified_at, Unset):
            raise NotPresentError(self, "modified_at")
        return self._modified_at

    @modified_at.setter
    def modified_at(self, value: datetime.datetime) -> None:
        self._modified_at = value

    @modified_at.deleter
    def modified_at(self) -> None:
        self._modified_at = UNSET

    @property
    def output_processor_id(self) -> str:
        if isinstance(self._output_processor_id, Unset):
            raise NotPresentError(self, "output_processor_id")
        return self._output_processor_id

    @output_processor_id.setter
    def output_processor_id(self, value: str) -> None:
        self._output_processor_id = value

    @output_processor_id.deleter
    def output_processor_id(self) -> None:
        self._output_processor_id = UNSET

    @property
    def status(self) -> LabAutomationTransformStatus:
        if isinstance(self._status, Unset):
            raise NotPresentError(self, "status")
        return self._status

    @status.setter
    def status(self, value: LabAutomationTransformStatus) -> None:
        self._status = value

    @status.deleter
    def status(self) -> None:
        self._status = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="AppConfigItemsArchivalChange")


@attr.s(auto_attribs=True, repr=False)
class AppConfigItemsArchivalChange:
    """ IDs of all app config items that were archived or unarchived. """

    _item_ids: Union[Unset, List[str]] = UNSET

    def __repr__(self):
        fields = []
        fields.append("item_ids={}".format(repr(self._item_ids)))
        return "AppConfigItemsArchivalChange({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        item_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._item_ids, Unset):
            item_ids = self._item_ids

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if item_ids is not UNSET:
            field_dict["itemIds"] = item_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_item_ids() -> Union[Unset, List[str]]:
            item_ids = cast(List[str], d.pop("itemIds"))

            return item_ids

        try:
            item_ids = get_item_ids()
        except KeyError:
            if strict:
                raise
            item_ids = cast(Union[Unset, List[str]], UNSET)

        app_config_items_archival_change = cls(
            item_ids=item_ids,
        )

        return app_config_items_archival_change

    @property
    def item_ids(self) -> List[str]:
        if isinstance(self._item_ids, Unset):
            raise NotPresentError(self, "item_ids")
        return self._item_ids

    @item_ids.setter
    def item_ids(self, value: List[str]) -> None:
        self._item_ids = value

    @item_ids.deleter
    def item_ids(self) -> None:
        self._item_ids = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowTaskGroupCreate")


@attr.s(auto_attribs=True, repr=False)
class WorkflowTaskGroupCreate:
    """  """

    _schema_id: str
    _folder_id: Union[Unset, str] = UNSET
    _name: Union[Unset, str] = UNSET
    _watcher_ids: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("schema_id={}".format(repr(self._schema_id)))
        fields.append("folder_id={}".format(repr(self._folder_id)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("watcher_ids={}".format(repr(self._watcher_ids)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "WorkflowTaskGroupCreate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        schema_id = self._schema_id
        folder_id = self._folder_id
        name = self._name
        watcher_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._watcher_ids, Unset):
            watcher_ids = self._watcher_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if schema_id is not UNSET:
            field_dict["schemaId"] = schema_id
        if folder_id is not UNSET:
            field_dict["folderId"] = folder_id
        if name is not UNSET:
            field_dict["name"] = name
        if watcher_ids is not UNSET:
            field_dict["watcherIds"] = watcher_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_schema_id() -> str:
            schema_id = d.pop("schemaId")
            return schema_id

        try:
            schema_id = get_schema_id()
        except KeyError:
            if strict:
                raise
            schema_id = cast(str, UNSET)

        def get_folder_id() -> Union[Unset, str]:
            folder_id = d.pop("folderId")
            return folder_id

        try:
            folder_id = get_folder_id()
        except KeyError:
            if strict:
                raise
            folder_id = cast(Union[Unset, str], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        def get_watcher_ids() -> Union[Unset, List[str]]:
            watcher_ids = cast(List[str], d.pop("watcherIds"))

            return watcher_ids

        try:
            watcher_ids = get_watcher_ids()
        except KeyError:
            if strict:
                raise
            watcher_ids = cast(Union[Unset, List[str]], UNSET)

        workflow_task_group_create = cls(
            schema_id=schema_id,
            folder_id=folder_id,
            name=name,
            watcher_ids=watcher_ids,
        )

        workflow_task_group_create.additional_properties = d
        return workflow_task_group_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def schema_id(self) -> str:
        """ The workflow task schema of tasks in this task group """
        if isinstance(self._schema_id, Unset):
            raise NotPresentError(self, "schema_id")
        return self._schema_id

    @schema_id.setter
    def schema_id(self, value: str) -> None:
        self._schema_id = value

    @property
    def folder_id(self) -> str:
        """ ID of the folder that contains the workflow task group """
        if isinstance(self._folder_id, Unset):
            raise NotPresentError(self, "folder_id")
        return self._folder_id

    @folder_id.setter
    def folder_id(self, value: str) -> None:
        self._folder_id = value

    @folder_id.deleter
    def folder_id(self) -> None:
        self._folder_id = UNSET

    @property
    def name(self) -> str:
        """ The name of the workflow task group """
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET

    @property
    def watcher_ids(self) -> List[str]:
        """ IDs of the users watching the workflow task group """
        if isinstance(self._watcher_ids, Unset):
            raise NotPresentError(self, "watcher_ids")
        return self._watcher_ids

    @watcher_ids.setter
    def watcher_ids(self, value: List[str]) -> None:
        self._watcher_ids = value

    @watcher_ids.deleter
    def watcher_ids(self) -> None:
        self._watcher_ids = UNSET



import datetime
from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.bad_request_error import BadRequestError
from ...models.list_sessions_sort import ListSessionsSort
from ...models.not_found_error import NotFoundError
from ...models.sessions_paginated_list import SessionsPaginatedList
from ...types import Response, UNSET, Unset


def _get_kwargs(
    *,
    client: Client,
    app_id: Union[Unset, str] = UNSET,
    sort: Union[Unset, ListSessionsSort] = ListSessionsSort.MODIFIEDAT,
    created_atlt: Union[Unset, datetime.datetime] = UNSET,
    created_atgt: Union[Unset, datetime.datetime] = UNSET,
    created_atlte: Union[Unset, datetime.datetime] = UNSET,
    created_atgte: Union[Unset, datetime.datetime] = UNSET,
    modified_atlt: Union[Unset, datetime.datetime] = UNSET,
    modified_atgt: Union[Unset, datetime.datetime] = UNSET,
    modified_atlte: Union[Unset, datetime.datetime] = UNSET,
    modified_atgte: Union[Unset, datetime.datetime] = UNSET,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
) -> Dict[str, Any]:
    url = "{}/sessions".format(client.base_url)

    headers: Dict[str, Any] = client.httpx_client.headers
    headers.update(client.get_headers())

    cookies: Dict[str, Any] = client.httpx_client.cookies
    cookies.update(client.get_cookies())

    json_sort: Union[Unset, int] = UNSET
    if not isinstance(sort, Unset):
        json_sort = sort.value

    json_created_atlt: Union[Unset, str] = UNSET
    if not isinstance(created_atlt, Unset):
        json_created_atlt = created_atlt.isoformat()

    json_created_atgt: Union[Unset, str] = UNSET
    if not isinstance(created_atgt, Unset):
        json_created_atgt = created_atgt.isoformat()

    json_created_atlte: Union[Unset, str] = UNSET
    if not isinstance(created_atlte, Unset):
        json_created_atlte = created_atlte.isoformat()

    json_created_atgte: Union[Unset, str] = UNSET
    if not isinstance(created_atgte, Unset):
        json_created_atgte = created_atgte.isoformat()

    json_modified_atlt: Union[Unset, str] = UNSET
    if not isinstance(modified_atlt, Unset):
        json_modified_atlt = modified_atlt.isoformat()

    json_modified_atgt: Union[Unset, str] = UNSET
    if not isinstance(modified_atgt, Unset):
        json_modified_atgt = modified_atgt.isoformat()

    json_modified_atlte: Union[Unset, str] = UNSET
    if not isinstance(modified_atlte, Unset):
        json_modified_atlte = modified_atlte.isoformat()

    json_modified_atgte: Union[Unset, str] = UNSET
    if not isinstance(modified_atgte, Unset):
        json_modified_atgte = modified_atgte.isoformat()

    params: Dict[str, Any] = {}
    if not isinstance(app_id, Unset) and app_id is not None:
        params["appId"] = app_id
    if not isinstance(json_sort, Unset) and json_sort is not None:
        params["sort"] = json_sort
    if not isinstance(json_created_atlt, Unset) and json_created_atlt is not None:
        params["createdAt.lt"] = json_created_atlt
    if not isinstance(json_created_atgt, Unset) and json_created_atgt is not None:
        params["createdAt.gt"] = json_created_atgt
    if not isinstance(json_created_atlte, Unset) and json_created_atlte is not None:
        params["createdAt.lte"] = json_created_atlte
    if not isinstance(json_created_atgte, Unset) and json_created_atgte is not None:
        params["createdAt.gte"] = json_created_atgte
    if not isinstance(json_modified_atlt, Unset) and json_modified_atlt is not None:
        params["modifiedAt.lt"] = json_modified_atlt
    if not isinstance(json_modified_atgt, Unset) and json_modified_atgt is not None:
        params["modifiedAt.gt"] = json_modified_atgt
    if not isinstance(json_modified_atlte, Unset) and json_modified_atlte is not None:
        params["modifiedAt.lte"] = json_modified_atlte
    if not isinstance(json_modified_atgte, Unset) and json_modified_atgte is not None:
        params["modifiedAt.gte"] = json_modified_atgte
    if not isinstance(next_token, Unset) and next_token is not None:
        params["nextToken"] = next_token
    if not isinstance(page_size, Unset) and page_size is not None:
        params["pageSize"] = page_size

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[SessionsPaginatedList, BadRequestError, NotFoundError]]:
    if response.status_code == 200:
        response_200 = SessionsPaginatedList.from_dict(response.json(), strict=False)

        return response_200
    if response.status_code == 400:
        response_400 = BadRequestError.from_dict(response.json(), strict=False)

        return response_400
    if response.status_code == 404:
        response_404 = NotFoundError.from_dict(response.json(), strict=False)

        return response_404
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[SessionsPaginatedList, BadRequestError, NotFoundError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    app_id: Union[Unset, str] = UNSET,
    sort: Union[Unset, ListSessionsSort] = ListSessionsSort.MODIFIEDAT,
    created_atlt: Union[Unset, datetime.datetime] = UNSET,
    created_atgt: Union[Unset, datetime.datetime] = UNSET,
    created_atlte: Union[Unset, datetime.datetime] = UNSET,
    created_atgte: Union[Unset, datetime.datetime] = UNSET,
    modified_atlt: Union[Unset, datetime.datetime] = UNSET,
    modified_atgt: Union[Unset, datetime.datetime] = UNSET,
    modified_atlte: Union[Unset, datetime.datetime] = UNSET,
    modified_atgte: Union[Unset, datetime.datetime] = UNSET,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
) -> Response[Union[SessionsPaginatedList, BadRequestError, NotFoundError]]:
    kwargs = _get_kwargs(
        client=client,
        app_id=app_id,
        sort=sort,
        created_atlt=created_atlt,
        created_atgt=created_atgt,
        created_atlte=created_atlte,
        created_atgte=created_atgte,
        modified_atlt=modified_atlt,
        modified_atgt=modified_atgt,
        modified_atlte=modified_atlte,
        modified_atgte=modified_atgte,
        next_token=next_token,
        page_size=page_size,
    )

    response = client.httpx_client.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    app_id: Union[Unset, str] = UNSET,
    sort: Union[Unset, ListSessionsSort] = ListSessionsSort.MODIFIEDAT,
    created_atlt: Union[Unset, datetime.datetime] = UNSET,
    created_atgt: Union[Unset, datetime.datetime] = UNSET,
    created_atlte: Union[Unset, datetime.datetime] = UNSET,
    created_atgte: Union[Unset, datetime.datetime] = UNSET,
    modified_atlt: Union[Unset, datetime.datetime] = UNSET,
    modified_atgt: Union[Unset, datetime.datetime] = UNSET,
    modified_atlte: Union[Unset, datetime.datetime] = UNSET,
    modified_atgte: Union[Unset, datetime.datetime] = UNSET,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
) -> Optional[Union[SessionsPaginatedList, BadRequestError, NotFoundError]]:
    """ List all sessions """

    return sync_detailed(
        client=client,
        app_id=app_id,
        sort=sort,
        created_atlt=created_atlt,
        created_atgt=created_atgt,
        created_atlte=created_atlte,
        created_atgte=created_atgte,
        modified_atlt=modified_atlt,
        modified_atgt=modified_atgt,
        modified_atlte=modified_atlte,
        modified_atgte=modified_atgte,
        next_token=next_token,
        page_size=page_size,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    app_id: Union[Unset, str] = UNSET,
    sort: Union[Unset, ListSessionsSort] = ListSessionsSort.MODIFIEDAT,
    created_atlt: Union[Unset, datetime.datetime] = UNSET,
    created_atgt: Union[Unset, datetime.datetime] = UNSET,
    created_atlte: Union[Unset, datetime.datetime] = UNSET,
    created_atgte: Union[Unset, datetime.datetime] = UNSET,
    modified_atlt: Union[Unset, datetime.datetime] = UNSET,
    modified_atgt: Union[Unset, datetime.datetime] = UNSET,
    modified_atlte: Union[Unset, datetime.datetime] = UNSET,
    modified_atgte: Union[Unset, datetime.datetime] = UNSET,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
) -> Response[Union[SessionsPaginatedList, BadRequestError, NotFoundError]]:
    kwargs = _get_kwargs(
        client=client,
        app_id=app_id,
        sort=sort,
        created_atlt=created_atlt,
        created_atgt=created_atgt,
        created_atlte=created_atlte,
        created_atgte=created_atgte,
        modified_atlt=modified_atlt,
        modified_atgt=modified_atgt,
        modified_atlte=modified_atlte,
        modified_atgte=modified_atgte,
        next_token=next_token,
        page_size=page_size,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    app_id: Union[Unset, str] = UNSET,
    sort: Union[Unset, ListSessionsSort] = ListSessionsSort.MODIFIEDAT,
    created_atlt: Union[Unset, datetime.datetime] = UNSET,
    created_atgt: Union[Unset, datetime.datetime] = UNSET,
    created_atlte: Union[Unset, datetime.datetime] = UNSET,
    created_atgte: Union[Unset, datetime.datetime] = UNSET,
    modified_atlt: Union[Unset, datetime.datetime] = UNSET,
    modified_atgt: Union[Unset, datetime.datetime] = UNSET,
    modified_atlte: Union[Unset, datetime.datetime] = UNSET,
    modified_atgte: Union[Unset, datetime.datetime] = UNSET,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
) -> Optional[Union[SessionsPaginatedList, BadRequestError, NotFoundError]]:
    """ List all sessions """

    return (
        await asyncio_detailed(
            client=client,
            app_id=app_id,
            sort=sort,
            created_atlt=created_atlt,
            created_atgt=created_atgt,
            created_atlte=created_atlte,
            created_atgte=created_atgte,
            modified_atlt=modified_atlt,
            modified_atgt=modified_atgt,
            modified_atlte=modified_atlte,
            modified_atgte=modified_atgte,
            next_token=next_token,
            page_size=page_size,
        )
    ).parsed



from typing import Any, cast, Dict, List, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..models.dropdown_option_update import DropdownOptionUpdate
from ..types import UNSET, Unset

T = TypeVar("T", bound="DropdownUpdate")


@attr.s(auto_attribs=True, repr=False)
class DropdownUpdate:
    """  """

    _options: List[DropdownOptionUpdate]

    def __repr__(self):
        fields = []
        fields.append("options={}".format(repr(self._options)))
        return "DropdownUpdate({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        options = []
        for options_item_data in self._options:
            options_item = options_item_data.to_dict()

            options.append(options_item)

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if options is not UNSET:
            field_dict["options"] = options

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_options() -> List[DropdownOptionUpdate]:
            options = []
            _options = d.pop("options")
            for options_item_data in _options:
                options_item = DropdownOptionUpdate.from_dict(options_item_data, strict=False)

                options.append(options_item)

            return options

        try:
            options = get_options()
        except KeyError:
            if strict:
                raise
            options = cast(List[DropdownOptionUpdate], UNSET)

        dropdown_update = cls(
            options=options,
        )

        return dropdown_update

    @property
    def options(self) -> List[DropdownOptionUpdate]:
        """ Options to set for the dropdown """
        if isinstance(self._options, Unset):
            raise NotPresentError(self, "options")
        return self._options

    @options.setter
    def options(self, value: List[DropdownOptionUpdate]) -> None:
        self._options = value



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.bad_request_error import BadRequestError
from ...models.list_oligos_sort import ListOligosSort
from ...models.oligos_paginated_list import OligosPaginatedList
from ...models.schema_fields_query_param import SchemaFieldsQueryParam
from ...types import Response, UNSET, Unset


def _get_kwargs(
    *,
    client: Client,
    page_size: Union[Unset, int] = 50,
    next_token: Union[Unset, str] = UNSET,
    sort: Union[Unset, ListOligosSort] = ListOligosSort.MODIFIEDATDESC,
    modified_at: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    bases: Union[Unset, str] = UNSET,
    folder_id: Union[Unset, str] = UNSET,
    mentioned_in: Union[Unset, str] = UNSET,
    project_id: Union[Unset, str] = UNSET,
    registry_id: Union[Unset, None, str] = UNSET,
    schema_id: Union[Unset, str] = UNSET,
    schema_fields: Union[Unset, SchemaFieldsQueryParam] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
    mentions: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    entity_registry_idsany_of: Union[Unset, str] = UNSET,
    namesany_of: Union[Unset, str] = UNSET,
    namesany_ofcase_sensitive: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/oligos".format(client.base_url)

    headers: Dict[str, Any] = client.httpx_client.headers
    headers.update(client.get_headers())

    cookies: Dict[str, Any] = client.httpx_client.cookies
    cookies.update(client.get_cookies())

    json_sort: Union[Unset, int] = UNSET
    if not isinstance(sort, Unset):
        json_sort = sort.value

    json_schema_fields: Union[Unset, Dict[str, Any]] = UNSET
    if not isinstance(schema_fields, Unset):
        json_schema_fields = schema_fields.to_dict()

    params: Dict[str, Any] = {}
    if not isinstance(page_size, Unset) and page_size is not None:
        params["pageSize"] = page_size
    if not isinstance(next_token, Unset) and next_token is not None:
        params["nextToken"] = next_token
    if not isinstance(json_sort, Unset) and json_sort is not None:
        params["sort"] = json_sort
    if not isinstance(modified_at, Unset) and modified_at is not None:
        params["modifiedAt"] = modified_at
    if not isinstance(name, Unset) and name is not None:
        params["name"] = name
    if not isinstance(bases, Unset) and bases is not None:
        params["bases"] = bases
    if not isinstance(folder_id, Unset) and folder_id is not None:
        params["folderId"] = folder_id
    if not isinstance(mentioned_in, Unset) and mentioned_in is not None:
        params["mentionedIn"] = mentioned_in
    if not isinstance(project_id, Unset) and project_id is not None:
        params["projectId"] = project_id
    if not isinstance(registry_id, Unset) and registry_id is not None:
        params["registryId"] = registry_id
    if not isinstance(schema_id, Unset) and schema_id is not None:
        params["schemaId"] = schema_id
    if not isinstance(json_schema_fields, Unset) and json_schema_fields is not None:
        params.update(json_schema_fields)
    if not isinstance(archive_reason, Unset) and archive_reason is not None:
        params["archiveReason"] = archive_reason
    if not isinstance(mentions, Unset) and mentions is not None:
        params["mentions"] = mentions
    if not isinstance(ids, Unset) and ids is not None:
        params["ids"] = ids
    if not isinstance(entity_registry_idsany_of, Unset) and entity_registry_idsany_of is not None:
        params["entityRegistryIds.anyOf"] = entity_registry_idsany_of
    if not isinstance(namesany_of, Unset) and namesany_of is not None:
        params["names.anyOf"] = namesany_of
    if not isinstance(namesany_ofcase_sensitive, Unset) and namesany_ofcase_sensitive is not None:
        params["names.anyOf.caseSensitive"] = namesany_ofcase_sensitive
    if not isinstance(creator_ids, Unset) and creator_ids is not None:
        params["creatorIds"] = creator_ids

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[OligosPaginatedList, BadRequestError]]:
    if response.status_code == 200:
        response_200 = OligosPaginatedList.from_dict(response.json(), strict=False)

        return response_200
    if response.status_code == 400:
        response_400 = BadRequestError.from_dict(response.json(), strict=False)

        return response_400
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[OligosPaginatedList, BadRequestError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    page_size: Union[Unset, int] = 50,
    next_token: Union[Unset, str] = UNSET,
    sort: Union[Unset, ListOligosSort] = ListOligosSort.MODIFIEDATDESC,
    modified_at: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    bases: Union[Unset, str] = UNSET,
    folder_id: Union[Unset, str] = UNSET,
    mentioned_in: Union[Unset, str] = UNSET,
    project_id: Union[Unset, str] = UNSET,
    registry_id: Union[Unset, None, str] = UNSET,
    schema_id: Union[Unset, str] = UNSET,
    schema_fields: Union[Unset, SchemaFieldsQueryParam] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
    mentions: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    entity_registry_idsany_of: Union[Unset, str] = UNSET,
    namesany_of: Union[Unset, str] = UNSET,
    namesany_ofcase_sensitive: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
) -> Response[Union[OligosPaginatedList, BadRequestError]]:
    kwargs = _get_kwargs(
        client=client,
        page_size=page_size,
        next_token=next_token,
        sort=sort,
        modified_at=modified_at,
        name=name,
        bases=bases,
        folder_id=folder_id,
        mentioned_in=mentioned_in,
        project_id=project_id,
        registry_id=registry_id,
        schema_id=schema_id,
        schema_fields=schema_fields,
        archive_reason=archive_reason,
        mentions=mentions,
        ids=ids,
        entity_registry_idsany_of=entity_registry_idsany_of,
        namesany_of=namesany_of,
        namesany_ofcase_sensitive=namesany_ofcase_sensitive,
        creator_ids=creator_ids,
    )

    response = client.httpx_client.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    page_size: Union[Unset, int] = 50,
    next_token: Union[Unset, str] = UNSET,
    sort: Union[Unset, ListOligosSort] = ListOligosSort.MODIFIEDATDESC,
    modified_at: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    bases: Union[Unset, str] = UNSET,
    folder_id: Union[Unset, str] = UNSET,
    mentioned_in: Union[Unset, str] = UNSET,
    project_id: Union[Unset, str] = UNSET,
    registry_id: Union[Unset, None, str] = UNSET,
    schema_id: Union[Unset, str] = UNSET,
    schema_fields: Union[Unset, SchemaFieldsQueryParam] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
    mentions: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    entity_registry_idsany_of: Union[Unset, str] = UNSET,
    namesany_of: Union[Unset, str] = UNSET,
    namesany_ofcase_sensitive: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
) -> Optional[Union[OligosPaginatedList, BadRequestError]]:
    """ List Oligos """

    return sync_detailed(
        client=client,
        page_size=page_size,
        next_token=next_token,
        sort=sort,
        modified_at=modified_at,
        name=name,
        bases=bases,
        folder_id=folder_id,
        mentioned_in=mentioned_in,
        project_id=project_id,
        registry_id=registry_id,
        schema_id=schema_id,
        schema_fields=schema_fields,
        archive_reason=archive_reason,
        mentions=mentions,
        ids=ids,
        entity_registry_idsany_of=entity_registry_idsany_of,
        namesany_of=namesany_of,
        namesany_ofcase_sensitive=namesany_ofcase_sensitive,
        creator_ids=creator_ids,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    page_size: Union[Unset, int] = 50,
    next_token: Union[Unset, str] = UNSET,
    sort: Union[Unset, ListOligosSort] = ListOligosSort.MODIFIEDATDESC,
    modified_at: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    bases: Union[Unset, str] = UNSET,
    folder_id: Union[Unset, str] = UNSET,
    mentioned_in: Union[Unset, str] = UNSET,
    project_id: Union[Unset, str] = UNSET,
    registry_id: Union[Unset, None, str] = UNSET,
    schema_id: Union[Unset, str] = UNSET,
    schema_fields: Union[Unset, SchemaFieldsQueryParam] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
    mentions: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    entity_registry_idsany_of: Union[Unset, str] = UNSET,
    namesany_of: Union[Unset, str] = UNSET,
    namesany_ofcase_sensitive: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
) -> Response[Union[OligosPaginatedList, BadRequestError]]:
    kwargs = _get_kwargs(
        client=client,
        page_size=page_size,
        next_token=next_token,
        sort=sort,
        modified_at=modified_at,
        name=name,
        bases=bases,
        folder_id=folder_id,
        mentioned_in=mentioned_in,
        project_id=project_id,
        registry_id=registry_id,
        schema_id=schema_id,
        schema_fields=schema_fields,
        archive_reason=archive_reason,
        mentions=mentions,
        ids=ids,
        entity_registry_idsany_of=entity_registry_idsany_of,
        namesany_of=namesany_of,
        namesany_ofcase_sensitive=namesany_ofcase_sensitive,
        creator_ids=creator_ids,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    page_size: Union[Unset, int] = 50,
    next_token: Union[Unset, str] = UNSET,
    sort: Union[Unset, ListOligosSort] = ListOligosSort.MODIFIEDATDESC,
    modified_at: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    bases: Union[Unset, str] = UNSET,
    folder_id: Union[Unset, str] = UNSET,
    mentioned_in: Union[Unset, str] = UNSET,
    project_id: Union[Unset, str] = UNSET,
    registry_id: Union[Unset, None, str] = UNSET,
    schema_id: Union[Unset, str] = UNSET,
    schema_fields: Union[Unset, SchemaFieldsQueryParam] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
    mentions: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    entity_registry_idsany_of: Union[Unset, str] = UNSET,
    namesany_of: Union[Unset, str] = UNSET,
    namesany_ofcase_sensitive: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
) -> Optional[Union[OligosPaginatedList, BadRequestError]]:
    """ List Oligos """

    return (
        await asyncio_detailed(
            client=client,
            page_size=page_size,
            next_token=next_token,
            sort=sort,
            modified_at=modified_at,
            name=name,
            bases=bases,
            folder_id=folder_id,
            mentioned_in=mentioned_in,
            project_id=project_id,
            registry_id=registry_id,
            schema_id=schema_id,
            schema_fields=schema_fields,
            archive_reason=archive_reason,
            mentions=mentions,
            ids=ids,
            entity_registry_idsany_of=entity_registry_idsany_of,
            namesany_of=namesany_of,
            namesany_ofcase_sensitive=namesany_ofcase_sensitive,
            creator_ids=creator_ids,
        )
    ).parsed



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.plate import Plate
from ..types import UNSET, Unset

T = TypeVar("T", bound="PlatesBulkGet")


@attr.s(auto_attribs=True, repr=False)
class PlatesBulkGet:
    """  """

    _plates: Union[Unset, List[Plate]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("plates={}".format(repr(self._plates)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "PlatesBulkGet({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        plates: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._plates, Unset):
            plates = []
            for plates_item_data in self._plates:
                plates_item = plates_item_data.to_dict()

                plates.append(plates_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if plates is not UNSET:
            field_dict["plates"] = plates

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_plates() -> Union[Unset, List[Plate]]:
            plates = []
            _plates = d.pop("plates")
            for plates_item_data in _plates or []:
                plates_item = Plate.from_dict(plates_item_data, strict=False)

                plates.append(plates_item)

            return plates

        try:
            plates = get_plates()
        except KeyError:
            if strict:
                raise
            plates = cast(Union[Unset, List[Plate]], UNSET)

        plates_bulk_get = cls(
            plates=plates,
        )

        plates_bulk_get.additional_properties = d
        return plates_bulk_get

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def plates(self) -> List[Plate]:
        if isinstance(self._plates, Unset):
            raise NotPresentError(self, "plates")
        return self._plates

    @plates.setter
    def plates(self, value: List[Plate]) -> None:
        self._plates = value

    @plates.deleter
    def plates(self) -> None:
        self._plates = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.fields import Fields
from ..models.schema_summary import SchemaSummary
from ..types import UNSET, Unset

T = TypeVar("T", bound="RequestTask")


@attr.s(auto_attribs=True, repr=False)
class RequestTask:
    """A request task."""

    _id: str
    _schema: Union[Unset, None, SchemaSummary] = UNSET
    _fields: Union[Unset, Fields] = UNSET
    _sample_group_ids: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("id={}".format(repr(self._id)))
        fields.append("schema={}".format(repr(self._schema)))
        fields.append("fields={}".format(repr(self._fields)))
        fields.append("sample_group_ids={}".format(repr(self._sample_group_ids)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "RequestTask({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        id = self._id
        schema: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._schema, Unset):
            schema = self._schema.to_dict() if self._schema else None

        fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._fields, Unset):
            fields = self._fields.to_dict()

        sample_group_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._sample_group_ids, Unset):
            sample_group_ids = self._sample_group_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if id is not UNSET:
            field_dict["id"] = id
        if schema is not UNSET:
            field_dict["schema"] = schema
        if fields is not UNSET:
            field_dict["fields"] = fields
        if sample_group_ids is not UNSET:
            field_dict["sampleGroupIds"] = sample_group_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_id() -> str:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(str, UNSET)

        def get_schema() -> Union[Unset, None, SchemaSummary]:
            schema = None
            _schema = d.pop("schema")

            if _schema is not None and not isinstance(_schema, Unset):
                schema = SchemaSummary.from_dict(_schema)

            return schema

        try:
            schema = get_schema()
        except KeyError:
            if strict:
                raise
            schema = cast(Union[Unset, None, SchemaSummary], UNSET)

        def get_fields() -> Union[Unset, Fields]:
            fields: Union[Unset, Union[Unset, Fields]] = UNSET
            _fields = d.pop("fields")

            if not isinstance(_fields, Unset):
                fields = Fields.from_dict(_fields)

            return fields

        try:
            fields = get_fields()
        except KeyError:
            if strict:
                raise
            fields = cast(Union[Unset, Fields], UNSET)

        def get_sample_group_ids() -> Union[Unset, List[str]]:
            sample_group_ids = cast(List[str], d.pop("sampleGroupIds"))

            return sample_group_ids

        try:
            sample_group_ids = get_sample_group_ids()
        except KeyError:
            if strict:
                raise
            sample_group_ids = cast(Union[Unset, List[str]], UNSET)

        request_task = cls(
            id=id,
            schema=schema,
            fields=fields,
            sample_group_ids=sample_group_ids,
        )

        request_task.additional_properties = d
        return request_task

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def id(self) -> str:
        """ ID of the request task """
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @property
    def schema(self) -> Optional[SchemaSummary]:
        if isinstance(self._schema, Unset):
            raise NotPresentError(self, "schema")
        return self._schema

    @schema.setter
    def schema(self, value: Optional[SchemaSummary]) -> None:
        self._schema = value

    @schema.deleter
    def schema(self) -> None:
        self._schema = UNSET

    @property
    def fields(self) -> Fields:
        if isinstance(self._fields, Unset):
            raise NotPresentError(self, "fields")
        return self._fields

    @fields.setter
    def fields(self, value: Fields) -> None:
        self._fields = value

    @fields.deleter
    def fields(self) -> None:
        self._fields = UNSET

    @property
    def sample_group_ids(self) -> List[str]:
        """ IDs of all request sample groups now associated with this task. """
        if isinstance(self._sample_group_ids, Unset):
            raise NotPresentError(self, "sample_group_ids")
        return self._sample_group_ids

    @sample_group_ids.setter
    def sample_group_ids(self, value: List[str]) -> None:
        self._sample_group_ids = value

    @sample_group_ids.deleter
    def sample_group_ids(self) -> None:
        self._sample_group_ids = UNSET



from typing import Any, cast, Dict, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.molecule_structure_structure_format import MoleculeStructureStructureFormat
from ..types import UNSET, Unset

T = TypeVar("T", bound="MoleculeStructure")


@attr.s(auto_attribs=True, repr=False)
class MoleculeStructure:
    """  """

    _structure_format: Union[Unset, MoleculeStructureStructureFormat] = UNSET
    _value: Union[Unset, str] = UNSET

    def __repr__(self):
        fields = []
        fields.append("structure_format={}".format(repr(self._structure_format)))
        fields.append("value={}".format(repr(self._value)))
        return "MoleculeStructure({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        structure_format: Union[Unset, int] = UNSET
        if not isinstance(self._structure_format, Unset):
            structure_format = self._structure_format.value

        value = self._value

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if structure_format is not UNSET:
            field_dict["structureFormat"] = structure_format
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_structure_format() -> Union[Unset, MoleculeStructureStructureFormat]:
            structure_format = UNSET
            _structure_format = d.pop("structureFormat")
            if _structure_format is not None and _structure_format is not UNSET:
                try:
                    structure_format = MoleculeStructureStructureFormat(_structure_format)
                except ValueError:
                    structure_format = MoleculeStructureStructureFormat.of_unknown(_structure_format)

            return structure_format

        try:
            structure_format = get_structure_format()
        except KeyError:
            if strict:
                raise
            structure_format = cast(Union[Unset, MoleculeStructureStructureFormat], UNSET)

        def get_value() -> Union[Unset, str]:
            value = d.pop("value")
            return value

        try:
            value = get_value()
        except KeyError:
            if strict:
                raise
            value = cast(Union[Unset, str], UNSET)

        molecule_structure = cls(
            structure_format=structure_format,
            value=value,
        )

        return molecule_structure

    @property
    def structure_format(self) -> MoleculeStructureStructureFormat:
        if isinstance(self._structure_format, Unset):
            raise NotPresentError(self, "structure_format")
        return self._structure_format

    @structure_format.setter
    def structure_format(self, value: MoleculeStructureStructureFormat) -> None:
        self._structure_format = value

    @structure_format.deleter
    def structure_format(self) -> None:
        self._structure_format = UNSET

    @property
    def value(self) -> str:
        """ Chemical structure in SMILES or molfile format. """
        if isinstance(self._value, Unset):
            raise NotPresentError(self, "value")
        return self._value

    @value.setter
    def value(self, value: str) -> None:
        self._value = value

    @value.deleter
    def value(self) -> None:
        self._value = UNSET



from typing import Any, cast, Dict, Optional, Union

import httpx

from ...client import Client
from ...extensions import UnknownType
from ...models.bad_request_error import BadRequestError
from ...models.batch_worklist_items_list import BatchWorklistItemsList
from ...models.container_worklist_items_list import ContainerWorklistItemsList
from ...models.entity_worklist_items_list import EntityWorklistItemsList
from ...models.plate_worklist_items_list import PlateWorklistItemsList
from ...types import Response, UNSET, Unset


def _get_kwargs(
    *,
    client: Client,
    worklist_id: str,
    page_size: Union[Unset, int] = 50,
    next_token: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/worklists/{worklist_id}/items".format(client.base_url, worklist_id=worklist_id)

    headers: Dict[str, Any] = client.httpx_client.headers
    headers.update(client.get_headers())

    cookies: Dict[str, Any] = client.httpx_client.cookies
    cookies.update(client.get_cookies())

    params: Dict[str, Any] = {}
    if not isinstance(page_size, Unset) and page_size is not None:
        params["pageSize"] = page_size
    if not isinstance(next_token, Unset) and next_token is not None:
        params["nextToken"] = next_token
    if not isinstance(archive_reason, Unset) and archive_reason is not None:
        params["archiveReason"] = archive_reason

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        Union[
            ContainerWorklistItemsList,
            EntityWorklistItemsList,
            PlateWorklistItemsList,
            BatchWorklistItemsList,
            UnknownType,
        ],
        BadRequestError,
    ]
]:
    if response.status_code == 200:

        def _parse_response_200(
            data: Union[Dict[str, Any]]
        ) -> Union[
            ContainerWorklistItemsList,
            EntityWorklistItemsList,
            PlateWorklistItemsList,
            BatchWorklistItemsList,
            UnknownType,
        ]:
            response_200: Union[
                ContainerWorklistItemsList,
                EntityWorklistItemsList,
                PlateWorklistItemsList,
                BatchWorklistItemsList,
                UnknownType,
            ]
            discriminator_value: str = cast(str, data.get("type"))
            if discriminator_value is not None:
                worklist_items_paginated_list: Union[
                    ContainerWorklistItemsList,
                    EntityWorklistItemsList,
                    PlateWorklistItemsList,
                    BatchWorklistItemsList,
                    UnknownType,
                ]
                if discriminator_value == "batch":
                    worklist_items_paginated_list = BatchWorklistItemsList.from_dict(data, strict=False)

                    return worklist_items_paginated_list
                if discriminator_value == "bioentity":
                    worklist_items_paginated_list = EntityWorklistItemsList.from_dict(data, strict=False)

                    return worklist_items_paginated_list
                if discriminator_value == "container":
                    worklist_items_paginated_list = ContainerWorklistItemsList.from_dict(data, strict=False)

                    return worklist_items_paginated_list
                if discriminator_value == "plate":
                    worklist_items_paginated_list = PlateWorklistItemsList.from_dict(data, strict=False)

                    return worklist_items_paginated_list

                return UnknownType(value=data)
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                worklist_items_paginated_list = ContainerWorklistItemsList.from_dict(data, strict=True)

                return worklist_items_paginated_list
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                worklist_items_paginated_list = EntityWorklistItemsList.from_dict(data, strict=True)

                return worklist_items_paginated_list
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                worklist_items_paginated_list = PlateWorklistItemsList.from_dict(data, strict=True)

                return worklist_items_paginated_list
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                worklist_items_paginated_list = BatchWorklistItemsList.from_dict(data, strict=True)

                return worklist_items_paginated_list
            except:  # noqa: E722
                pass
            return UnknownType(data)

        response_200 = _parse_response_200(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = BadRequestError.from_dict(response.json(), strict=False)

        return response_400
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[
        Union[
            ContainerWorklistItemsList,
            EntityWorklistItemsList,
            PlateWorklistItemsList,
            BatchWorklistItemsList,
            UnknownType,
        ],
        BadRequestError,
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    worklist_id: str,
    page_size: Union[Unset, int] = 50,
    next_token: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
) -> Response[
    Union[
        Union[
            ContainerWorklistItemsList,
            EntityWorklistItemsList,
            PlateWorklistItemsList,
            BatchWorklistItemsList,
            UnknownType,
        ],
        BadRequestError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        worklist_id=worklist_id,
        page_size=page_size,
        next_token=next_token,
        archive_reason=archive_reason,
    )

    response = client.httpx_client.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    worklist_id: str,
    page_size: Union[Unset, int] = 50,
    next_token: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
) -> Optional[
    Union[
        Union[
            ContainerWorklistItemsList,
            EntityWorklistItemsList,
            PlateWorklistItemsList,
            BatchWorklistItemsList,
            UnknownType,
        ],
        BadRequestError,
    ]
]:
    """List items in a worklist. Items are ordered by their position within the worklist."""

    return sync_detailed(
        client=client,
        worklist_id=worklist_id,
        page_size=page_size,
        next_token=next_token,
        archive_reason=archive_reason,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    worklist_id: str,
    page_size: Union[Unset, int] = 50,
    next_token: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
) -> Response[
    Union[
        Union[
            ContainerWorklistItemsList,
            EntityWorklistItemsList,
            PlateWorklistItemsList,
            BatchWorklistItemsList,
            UnknownType,
        ],
        BadRequestError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        worklist_id=worklist_id,
        page_size=page_size,
        next_token=next_token,
        archive_reason=archive_reason,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    worklist_id: str,
    page_size: Union[Unset, int] = 50,
    next_token: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
) -> Optional[
    Union[
        Union[
            ContainerWorklistItemsList,
            EntityWorklistItemsList,
            PlateWorklistItemsList,
            BatchWorklistItemsList,
            UnknownType,
        ],
        BadRequestError,
    ]
]:
    """List items in a worklist. Items are ordered by their position within the worklist."""

    return (
        await asyncio_detailed(
            client=client,
            worklist_id=worklist_id,
            page_size=page_size,
            next_token=next_token,
            archive_reason=archive_reason,
        )
    ).parsed



from enum import Enum
from functools import lru_cache
from typing import cast

from ..extensions import Enums


class WorkflowTaskExecutionOriginType(Enums.KnownString):
    API = "API"
    ENTRY = "ENTRY"
    MODAL = "MODAL"

    def __str__(self) -> str:
        return str(self.value)

    @staticmethod
    @lru_cache(maxsize=None)
    def of_unknown(val: str) -> "WorkflowTaskExecutionOriginType":
        if not isinstance(val, str):
            raise ValueError(
                f"Value of WorkflowTaskExecutionOriginType must be a string (encountered: {val})"
            )
        newcls = Enum("WorkflowTaskExecutionOriginType", {"_UNKNOWN": val}, type=Enums.UnknownString)  # type: ignore
        return cast(WorkflowTaskExecutionOriginType, getattr(newcls, "_UNKNOWN"))



import datetime
from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..extensions import NotPresentError
from ..models.archive_record import ArchiveRecord
from ..models.assay_result_field_validation import AssayResultFieldValidation
from ..models.fields import Fields
from ..models.schema_summary import SchemaSummary
from ..models.user_summary import UserSummary
from ..types import UNSET, Unset

T = TypeVar("T", bound="AssayResult")


@attr.s(auto_attribs=True, repr=False)
class AssayResult:
    """  """

    _archive_record: Union[Unset, None, ArchiveRecord] = UNSET
    _created_at: Union[Unset, datetime.datetime] = UNSET
    _creator: Union[Unset, UserSummary] = UNSET
    _entry_id: Union[Unset, None, str] = UNSET
    _field_validation: Union[Unset, AssayResultFieldValidation] = UNSET
    _fields: Union[Unset, Fields] = UNSET
    _id: Union[Unset, str] = UNSET
    _is_reviewed: Union[Unset, bool] = UNSET
    _modified_at: Union[Unset, datetime.datetime] = UNSET
    _project_id: Union[Unset, None, str] = UNSET
    _schema: Union[Unset, SchemaSummary] = UNSET
    _validation_comment: Union[Unset, str] = UNSET
    _validation_status: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("archive_record={}".format(repr(self._archive_record)))
        fields.append("created_at={}".format(repr(self._created_at)))
        fields.append("creator={}".format(repr(self._creator)))
        fields.append("entry_id={}".format(repr(self._entry_id)))
        fields.append("field_validation={}".format(repr(self._field_validation)))
        fields.append("fields={}".format(repr(self._fields)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("is_reviewed={}".format(repr(self._is_reviewed)))
        fields.append("modified_at={}".format(repr(self._modified_at)))
        fields.append("project_id={}".format(repr(self._project_id)))
        fields.append("schema={}".format(repr(self._schema)))
        fields.append("validation_comment={}".format(repr(self._validation_comment)))
        fields.append("validation_status={}".format(repr(self._validation_status)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "AssayResult({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        archive_record: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._archive_record, Unset):
            archive_record = self._archive_record.to_dict() if self._archive_record else None

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self._created_at, Unset):
            created_at = self._created_at.isoformat()

        creator: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._creator, Unset):
            creator = self._creator.to_dict()

        entry_id = self._entry_id
        field_validation: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._field_validation, Unset):
            field_validation = self._field_validation.to_dict()

        fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._fields, Unset):
            fields = self._fields.to_dict()

        id = self._id
        is_reviewed = self._is_reviewed
        modified_at: Union[Unset, str] = UNSET
        if not isinstance(self._modified_at, Unset):
            modified_at = self._modified_at.isoformat()

        project_id = self._project_id
        schema: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self._schema, Unset):
            schema = self._schema.to_dict()

        validation_comment = self._validation_comment
        validation_status = self._validation_status

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if archive_record is not UNSET:
            field_dict["archiveRecord"] = archive_record
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if creator is not UNSET:
            field_dict["creator"] = creator
        if entry_id is not UNSET:
            field_dict["entryId"] = entry_id
        if field_validation is not UNSET:
            field_dict["fieldValidation"] = field_validation
        if fields is not UNSET:
            field_dict["fields"] = fields
        if id is not UNSET:
            field_dict["id"] = id
        if is_reviewed is not UNSET:
            field_dict["isReviewed"] = is_reviewed
        if modified_at is not UNSET:
            field_dict["modifiedAt"] = modified_at
        if project_id is not UNSET:
            field_dict["projectId"] = project_id
        if schema is not UNSET:
            field_dict["schema"] = schema
        if validation_comment is not UNSET:
            field_dict["validationComment"] = validation_comment
        if validation_status is not UNSET:
            field_dict["validationStatus"] = validation_status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_archive_record() -> Union[Unset, None, ArchiveRecord]:
            archive_record = None
            _archive_record = d.pop("archiveRecord")

            if _archive_record is not None and not isinstance(_archive_record, Unset):
                archive_record = ArchiveRecord.from_dict(_archive_record)

            return archive_record

        try:
            archive_record = get_archive_record()
        except KeyError:
            if strict:
                raise
            archive_record = cast(Union[Unset, None, ArchiveRecord], UNSET)

        def get_created_at() -> Union[Unset, datetime.datetime]:
            created_at: Union[Unset, datetime.datetime] = UNSET
            _created_at = d.pop("createdAt")
            if _created_at is not None and not isinstance(_created_at, Unset):
                created_at = isoparse(cast(str, _created_at))

            return created_at

        try:
            created_at = get_created_at()
        except KeyError:
            if strict:
                raise
            created_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_creator() -> Union[Unset, UserSummary]:
            creator: Union[Unset, Union[Unset, UserSummary]] = UNSET
            _creator = d.pop("creator")

            if not isinstance(_creator, Unset):
                creator = UserSummary.from_dict(_creator)

            return creator

        try:
            creator = get_creator()
        except KeyError:
            if strict:
                raise
            creator = cast(Union[Unset, UserSummary], UNSET)

        def get_entry_id() -> Union[Unset, None, str]:
            entry_id = d.pop("entryId")
            return entry_id

        try:
            entry_id = get_entry_id()
        except KeyError:
            if strict:
                raise
            entry_id = cast(Union[Unset, None, str], UNSET)

        def get_field_validation() -> Union[Unset, AssayResultFieldValidation]:
            field_validation: Union[Unset, Union[Unset, AssayResultFieldValidation]] = UNSET
            _field_validation = d.pop("fieldValidation")

            if not isinstance(_field_validation, Unset):
                field_validation = AssayResultFieldValidation.from_dict(_field_validation)

            return field_validation

        try:
            field_validation = get_field_validation()
        except KeyError:
            if strict:
                raise
            field_validation = cast(Union[Unset, AssayResultFieldValidation], UNSET)

        def get_fields() -> Union[Unset, Fields]:
            fields: Union[Unset, Union[Unset, Fields]] = UNSET
            _fields = d.pop("fields")

            if not isinstance(_fields, Unset):
                fields = Fields.from_dict(_fields)

            return fields

        try:
            fields = get_fields()
        except KeyError:
            if strict:
                raise
            fields = cast(Union[Unset, Fields], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_is_reviewed() -> Union[Unset, bool]:
            is_reviewed = d.pop("isReviewed")
            return is_reviewed

        try:
            is_reviewed = get_is_reviewed()
        except KeyError:
            if strict:
                raise
            is_reviewed = cast(Union[Unset, bool], UNSET)

        def get_modified_at() -> Union[Unset, datetime.datetime]:
            modified_at: Union[Unset, datetime.datetime] = UNSET
            _modified_at = d.pop("modifiedAt")
            if _modified_at is not None and not isinstance(_modified_at, Unset):
                modified_at = isoparse(cast(str, _modified_at))

            return modified_at

        try:
            modified_at = get_modified_at()
        except KeyError:
            if strict:
                raise
            modified_at = cast(Union[Unset, datetime.datetime], UNSET)

        def get_project_id() -> Union[Unset, None, str]:
            project_id = d.pop("projectId")
            return project_id

        try:
            project_id = get_project_id()
        except KeyError:
            if strict:
                raise
            project_id = cast(Union[Unset, None, str], UNSET)

        def get_schema() -> Union[Unset, SchemaSummary]:
            schema: Union[Unset, Union[Unset, SchemaSummary]] = UNSET
            _schema = d.pop("schema")

            if not isinstance(_schema, Unset):
                schema = SchemaSummary.from_dict(_schema)

            return schema

        try:
            schema = get_schema()
        except KeyError:
            if strict:
                raise
            schema = cast(Union[Unset, SchemaSummary], UNSET)

        def get_validation_comment() -> Union[Unset, str]:
            validation_comment = d.pop("validationComment")
            return validation_comment

        try:
            validation_comment = get_validation_comment()
        except KeyError:
            if strict:
                raise
            validation_comment = cast(Union[Unset, str], UNSET)

        def get_validation_status() -> Union[Unset, str]:
            validation_status = d.pop("validationStatus")
            return validation_status

        try:
            validation_status = get_validation_status()
        except KeyError:
            if strict:
                raise
            validation_status = cast(Union[Unset, str], UNSET)

        assay_result = cls(
            archive_record=archive_record,
            created_at=created_at,
            creator=creator,
            entry_id=entry_id,
            field_validation=field_validation,
            fields=fields,
            id=id,
            is_reviewed=is_reviewed,
            modified_at=modified_at,
            project_id=project_id,
            schema=schema,
            validation_comment=validation_comment,
            validation_status=validation_status,
        )

        assay_result.additional_properties = d
        return assay_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def archive_record(self) -> Optional[ArchiveRecord]:
        if isinstance(self._archive_record, Unset):
            raise NotPresentError(self, "archive_record")
        return self._archive_record

    @archive_record.setter
    def archive_record(self, value: Optional[ArchiveRecord]) -> None:
        self._archive_record = value

    @archive_record.deleter
    def archive_record(self) -> None:
        self._archive_record = UNSET

    @property
    def created_at(self) -> datetime.datetime:
        """ DateTime at which the the result was created """
        if isinstance(self._created_at, Unset):
            raise NotPresentError(self, "created_at")
        return self._created_at

    @created_at.setter
    def created_at(self, value: datetime.datetime) -> None:
        self._created_at = value

    @created_at.deleter
    def created_at(self) -> None:
        self._created_at = UNSET

    @property
    def creator(self) -> UserSummary:
        if isinstance(self._creator, Unset):
            raise NotPresentError(self, "creator")
        return self._creator

    @creator.setter
    def creator(self, value: UserSummary) -> None:
        self._creator = value

    @creator.deleter
    def creator(self) -> None:
        self._creator = UNSET

    @property
    def entry_id(self) -> Optional[str]:
        """ ID of the entry that this result is attached to """
        if isinstance(self._entry_id, Unset):
            raise NotPresentError(self, "entry_id")
        return self._entry_id

    @entry_id.setter
    def entry_id(self, value: Optional[str]) -> None:
        self._entry_id = value

    @entry_id.deleter
    def entry_id(self) -> None:
        self._entry_id = UNSET

    @property
    def field_validation(self) -> AssayResultFieldValidation:
        """Object mapping field names to a UserValidation Resource object for that field. To **set** validation for a result, you *must* use this object."""
        if isinstance(self._field_validation, Unset):
            raise NotPresentError(self, "field_validation")
        return self._field_validation

    @field_validation.setter
    def field_validation(self, value: AssayResultFieldValidation) -> None:
        self._field_validation = value

    @field_validation.deleter
    def field_validation(self) -> None:
        self._field_validation = UNSET

    @property
    def fields(self) -> Fields:
        if isinstance(self._fields, Unset):
            raise NotPresentError(self, "fields")
        return self._fields

    @fields.setter
    def fields(self, value: Fields) -> None:
        self._fields = value

    @fields.deleter
    def fields(self) -> None:
        self._fields = UNSET

    @property
    def id(self) -> str:
        """ ID of the result """
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def is_reviewed(self) -> bool:
        """ Whether or not this result is attached to an accepted entry """
        if isinstance(self._is_reviewed, Unset):
            raise NotPresentError(self, "is_reviewed")
        return self._is_reviewed

    @is_reviewed.setter
    def is_reviewed(self, value: bool) -> None:
        self._is_reviewed = value

    @is_reviewed.deleter
    def is_reviewed(self) -> None:
        self._is_reviewed = UNSET

    @property
    def modified_at(self) -> datetime.datetime:
        """ DateTime at which the the result was last modified """
        if isinstance(self._modified_at, Unset):
            raise NotPresentError(self, "modified_at")
        return self._modified_at

    @modified_at.setter
    def modified_at(self, value: datetime.datetime) -> None:
        self._modified_at = value

    @modified_at.deleter
    def modified_at(self) -> None:
        self._modified_at = UNSET

    @property
    def project_id(self) -> Optional[str]:
        """ ID of the project to insert the result into """
        if isinstance(self._project_id, Unset):
            raise NotPresentError(self, "project_id")
        return self._project_id

    @project_id.setter
    def project_id(self, value: Optional[str]) -> None:
        self._project_id = value

    @project_id.deleter
    def project_id(self) -> None:
        self._project_id = UNSET

    @property
    def schema(self) -> SchemaSummary:
        if isinstance(self._schema, Unset):
            raise NotPresentError(self, "schema")
        return self._schema

    @schema.setter
    def schema(self, value: SchemaSummary) -> None:
        self._schema = value

    @schema.deleter
    def schema(self) -> None:
        self._schema = UNSET

    @property
    def validation_comment(self) -> str:
        if isinstance(self._validation_comment, Unset):
            raise NotPresentError(self, "validation_comment")
        return self._validation_comment

    @validation_comment.setter
    def validation_comment(self, value: str) -> None:
        self._validation_comment = value

    @validation_comment.deleter
    def validation_comment(self) -> None:
        self._validation_comment = UNSET

    @property
    def validation_status(self) -> str:
        if isinstance(self._validation_status, Unset):
            raise NotPresentError(self, "validation_status")
        return self._validation_status

    @validation_status.setter
    def validation_status(self, value: str) -> None:
        self._validation_status = value

    @validation_status.deleter
    def validation_status(self) -> None:
        self._validation_status = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.archive_record import ArchiveRecord
from ..models.schema_link_field_definition_type import SchemaLinkFieldDefinitionType
from ..types import UNSET, Unset

T = TypeVar("T", bound="SchemaLinkFieldDefinition")


@attr.s(auto_attribs=True, repr=False)
class SchemaLinkFieldDefinition:
    """  """

    _schema_id: Union[Unset, None, str] = UNSET
    _type: Union[Unset, SchemaLinkFieldDefinitionType] = UNSET
    _archive_record: Union[Unset, None, ArchiveRecord] = UNSET
    _id: Union[Unset, str] = UNSET
    _is_multi: Union[Unset, bool] = UNSET
    _is_required: Union[Unset, bool] = UNSET
    _name: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("schema_id={}".format(repr(self._schema_id)))
        fields.append("type={}".format(repr(self._type)))
        fields.append("archive_record={}".format(repr(self._archive_record)))
        fields.append("id={}".format(repr(self._id)))
        fields.append("is_multi={}".format(repr(self._is_multi)))
        fields.append("is_required={}".format(repr(self._is_required)))
        fields.append("name={}".format(repr(self._name)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "SchemaLinkFieldDefinition({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        schema_id = self._schema_id
        type: Union[Unset, int] = UNSET
        if not isinstance(self._type, Unset):
            type = self._type.value

        archive_record: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self._archive_record, Unset):
            archive_record = self._archive_record.to_dict() if self._archive_record else None

        id = self._id
        is_multi = self._is_multi
        is_required = self._is_required
        name = self._name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if schema_id is not UNSET:
            field_dict["schemaId"] = schema_id
        if type is not UNSET:
            field_dict["type"] = type
        if archive_record is not UNSET:
            field_dict["archiveRecord"] = archive_record
        if id is not UNSET:
            field_dict["id"] = id
        if is_multi is not UNSET:
            field_dict["isMulti"] = is_multi
        if is_required is not UNSET:
            field_dict["isRequired"] = is_required
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_schema_id() -> Union[Unset, None, str]:
            schema_id = d.pop("schemaId")
            return schema_id

        try:
            schema_id = get_schema_id()
        except KeyError:
            if strict:
                raise
            schema_id = cast(Union[Unset, None, str], UNSET)

        def get_type() -> Union[Unset, SchemaLinkFieldDefinitionType]:
            type = UNSET
            _type = d.pop("type")
            if _type is not None and _type is not UNSET:
                try:
                    type = SchemaLinkFieldDefinitionType(_type)
                except ValueError:
                    type = SchemaLinkFieldDefinitionType.of_unknown(_type)

            return type

        try:
            type = get_type()
        except KeyError:
            if strict:
                raise
            type = cast(Union[Unset, SchemaLinkFieldDefinitionType], UNSET)

        def get_archive_record() -> Union[Unset, None, ArchiveRecord]:
            archive_record = None
            _archive_record = d.pop("archiveRecord")

            if _archive_record is not None and not isinstance(_archive_record, Unset):
                archive_record = ArchiveRecord.from_dict(_archive_record)

            return archive_record

        try:
            archive_record = get_archive_record()
        except KeyError:
            if strict:
                raise
            archive_record = cast(Union[Unset, None, ArchiveRecord], UNSET)

        def get_id() -> Union[Unset, str]:
            id = d.pop("id")
            return id

        try:
            id = get_id()
        except KeyError:
            if strict:
                raise
            id = cast(Union[Unset, str], UNSET)

        def get_is_multi() -> Union[Unset, bool]:
            is_multi = d.pop("isMulti")
            return is_multi

        try:
            is_multi = get_is_multi()
        except KeyError:
            if strict:
                raise
            is_multi = cast(Union[Unset, bool], UNSET)

        def get_is_required() -> Union[Unset, bool]:
            is_required = d.pop("isRequired")
            return is_required

        try:
            is_required = get_is_required()
        except KeyError:
            if strict:
                raise
            is_required = cast(Union[Unset, bool], UNSET)

        def get_name() -> Union[Unset, str]:
            name = d.pop("name")
            return name

        try:
            name = get_name()
        except KeyError:
            if strict:
                raise
            name = cast(Union[Unset, str], UNSET)

        schema_link_field_definition = cls(
            schema_id=schema_id,
            type=type,
            archive_record=archive_record,
            id=id,
            is_multi=is_multi,
            is_required=is_required,
            name=name,
        )

        schema_link_field_definition.additional_properties = d
        return schema_link_field_definition

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def schema_id(self) -> Optional[str]:
        if isinstance(self._schema_id, Unset):
            raise NotPresentError(self, "schema_id")
        return self._schema_id

    @schema_id.setter
    def schema_id(self, value: Optional[str]) -> None:
        self._schema_id = value

    @schema_id.deleter
    def schema_id(self) -> None:
        self._schema_id = UNSET

    @property
    def type(self) -> SchemaLinkFieldDefinitionType:
        if isinstance(self._type, Unset):
            raise NotPresentError(self, "type")
        return self._type

    @type.setter
    def type(self, value: SchemaLinkFieldDefinitionType) -> None:
        self._type = value

    @type.deleter
    def type(self) -> None:
        self._type = UNSET

    @property
    def archive_record(self) -> Optional[ArchiveRecord]:
        if isinstance(self._archive_record, Unset):
            raise NotPresentError(self, "archive_record")
        return self._archive_record

    @archive_record.setter
    def archive_record(self, value: Optional[ArchiveRecord]) -> None:
        self._archive_record = value

    @archive_record.deleter
    def archive_record(self) -> None:
        self._archive_record = UNSET

    @property
    def id(self) -> str:
        if isinstance(self._id, Unset):
            raise NotPresentError(self, "id")
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @id.deleter
    def id(self) -> None:
        self._id = UNSET

    @property
    def is_multi(self) -> bool:
        if isinstance(self._is_multi, Unset):
            raise NotPresentError(self, "is_multi")
        return self._is_multi

    @is_multi.setter
    def is_multi(self, value: bool) -> None:
        self._is_multi = value

    @is_multi.deleter
    def is_multi(self) -> None:
        self._is_multi = UNSET

    @property
    def is_required(self) -> bool:
        if isinstance(self._is_required, Unset):
            raise NotPresentError(self, "is_required")
        return self._is_required

    @is_required.setter
    def is_required(self, value: bool) -> None:
        self._is_required = value

    @is_required.deleter
    def is_required(self) -> None:
        self._is_required = UNSET

    @property
    def name(self) -> str:
        if isinstance(self._name, Unset):
            raise NotPresentError(self, "name")
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @name.deleter
    def name(self) -> None:
        self._name = UNSET



from typing import Any, cast, Dict, List, Type, TypeVar

import attr

from ..extensions import NotPresentError
from ..models.plates_archive_reason import PlatesArchiveReason
from ..types import UNSET, Unset

T = TypeVar("T", bound="PlatesArchive")


@attr.s(auto_attribs=True, repr=False)
class PlatesArchive:
    """  """

    _plate_ids: List[str]
    _reason: PlatesArchiveReason
    _should_remove_barcodes: bool

    def __repr__(self):
        fields = []
        fields.append("plate_ids={}".format(repr(self._plate_ids)))
        fields.append("reason={}".format(repr(self._reason)))
        fields.append("should_remove_barcodes={}".format(repr(self._should_remove_barcodes)))
        return "PlatesArchive({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        plate_ids = self._plate_ids

        reason = self._reason.value

        should_remove_barcodes = self._should_remove_barcodes

        field_dict: Dict[str, Any] = {}
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if plate_ids is not UNSET:
            field_dict["plateIds"] = plate_ids
        if reason is not UNSET:
            field_dict["reason"] = reason
        if should_remove_barcodes is not UNSET:
            field_dict["shouldRemoveBarcodes"] = should_remove_barcodes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_plate_ids() -> List[str]:
            plate_ids = cast(List[str], d.pop("plateIds"))

            return plate_ids

        try:
            plate_ids = get_plate_ids()
        except KeyError:
            if strict:
                raise
            plate_ids = cast(List[str], UNSET)

        def get_reason() -> PlatesArchiveReason:
            _reason = d.pop("reason")
            try:
                reason = PlatesArchiveReason(_reason)
            except ValueError:
                reason = PlatesArchiveReason.of_unknown(_reason)

            return reason

        try:
            reason = get_reason()
        except KeyError:
            if strict:
                raise
            reason = cast(PlatesArchiveReason, UNSET)

        def get_should_remove_barcodes() -> bool:
            should_remove_barcodes = d.pop("shouldRemoveBarcodes")
            return should_remove_barcodes

        try:
            should_remove_barcodes = get_should_remove_barcodes()
        except KeyError:
            if strict:
                raise
            should_remove_barcodes = cast(bool, UNSET)

        plates_archive = cls(
            plate_ids=plate_ids,
            reason=reason,
            should_remove_barcodes=should_remove_barcodes,
        )

        return plates_archive

    @property
    def plate_ids(self) -> List[str]:
        """ Array of plate IDs """
        if isinstance(self._plate_ids, Unset):
            raise NotPresentError(self, "plate_ids")
        return self._plate_ids

    @plate_ids.setter
    def plate_ids(self, value: List[str]) -> None:
        self._plate_ids = value

    @property
    def reason(self) -> PlatesArchiveReason:
        """Reason that plates are being archived."""
        if isinstance(self._reason, Unset):
            raise NotPresentError(self, "reason")
        return self._reason

    @reason.setter
    def reason(self, value: PlatesArchiveReason) -> None:
        self._reason = value

    @property
    def should_remove_barcodes(self) -> bool:
        """Remove barcodes. Removing barcodes from archived inventory that contain items will also remove barcodes from the contained items."""
        if isinstance(self._should_remove_barcodes, Unset):
            raise NotPresentError(self, "should_remove_barcodes")
        return self._should_remove_barcodes

    @should_remove_barcodes.setter
    def should_remove_barcodes(self, value: bool) -> None:
        self._should_remove_barcodes = value



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.container_content import ContainerContent
from ..types import UNSET, Unset

T = TypeVar("T", bound="ContainerContentsList")


@attr.s(auto_attribs=True, repr=False)
class ContainerContentsList:
    """  """

    _contents: Union[Unset, List[ContainerContent]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("contents={}".format(repr(self._contents)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "ContainerContentsList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        contents: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._contents, Unset):
            contents = []
            for contents_item_data in self._contents:
                contents_item = contents_item_data.to_dict()

                contents.append(contents_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if contents is not UNSET:
            field_dict["contents"] = contents

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_contents() -> Union[Unset, List[ContainerContent]]:
            contents = []
            _contents = d.pop("contents")
            for contents_item_data in _contents or []:
                contents_item = ContainerContent.from_dict(contents_item_data, strict=False)

                contents.append(contents_item)

            return contents

        try:
            contents = get_contents()
        except KeyError:
            if strict:
                raise
            contents = cast(Union[Unset, List[ContainerContent]], UNSET)

        container_contents_list = cls(
            contents=contents,
        )

        container_contents_list.additional_properties = d
        return container_contents_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def contents(self) -> List[ContainerContent]:
        if isinstance(self._contents, Unset):
            raise NotPresentError(self, "contents")
        return self._contents

    @contents.setter
    def contents(self, value: List[ContainerContent]) -> None:
        self._contents = value

    @contents.deleter
    def contents(self) -> None:
        self._contents = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError, UnknownType
from ..models.dna_oligo import DnaOligo
from ..models.rna_oligo import RnaOligo
from ..types import UNSET, Unset

T = TypeVar("T", bound="OligosPaginatedList")


@attr.s(auto_attribs=True, repr=False)
class OligosPaginatedList:
    """  """

    _oligos: Union[Unset, List[Union[DnaOligo, RnaOligo, UnknownType]]] = UNSET
    _next_token: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("oligos={}".format(repr(self._oligos)))
        fields.append("next_token={}".format(repr(self._next_token)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "OligosPaginatedList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        oligos: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._oligos, Unset):
            oligos = []
            for oligos_item_data in self._oligos:
                oligos_item = oligos_item_data.to_dict()

                oligos.append(oligos_item)

        next_token = self._next_token

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if oligos is not UNSET:
            field_dict["oligos"] = oligos
        if next_token is not UNSET:
            field_dict["nextToken"] = next_token

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_oligos() -> Union[Unset, List[Union[DnaOligo, RnaOligo, UnknownType]]]:
            oligos = []
            _oligos = d.pop("oligos")
            for oligos_item_data in _oligos or []:
                oligos_item: Union[DnaOligo, RnaOligo, UnknownType]
                _oligos_item = oligos_item_data

                if True:
                    discriminator = _oligos_item["nucleotideType"]
                    if discriminator == "DNA":
                        oligos_item = DnaOligo.from_dict(_oligos_item)
                    elif discriminator == "RNA":
                        oligos_item = RnaOligo.from_dict(_oligos_item)
                    else:
                        oligos_item = UnknownType(value=_oligos_item)

                oligos.append(oligos_item)

            return oligos

        try:
            oligos = get_oligos()
        except KeyError:
            if strict:
                raise
            oligos = cast(Union[Unset, List[Union[DnaOligo, RnaOligo, UnknownType]]], UNSET)

        def get_next_token() -> Union[Unset, str]:
            next_token = d.pop("nextToken")
            return next_token

        try:
            next_token = get_next_token()
        except KeyError:
            if strict:
                raise
            next_token = cast(Union[Unset, str], UNSET)

        oligos_paginated_list = cls(
            oligos=oligos,
            next_token=next_token,
        )

        oligos_paginated_list.additional_properties = d
        return oligos_paginated_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def oligos(self) -> List[Union[DnaOligo, RnaOligo, UnknownType]]:
        if isinstance(self._oligos, Unset):
            raise NotPresentError(self, "oligos")
        return self._oligos

    @oligos.setter
    def oligos(self, value: List[Union[DnaOligo, RnaOligo, UnknownType]]) -> None:
        self._oligos = value

    @oligos.deleter
    def oligos(self) -> None:
        self._oligos = UNSET

    @property
    def next_token(self) -> str:
        if isinstance(self._next_token, Unset):
            raise NotPresentError(self, "next_token")
        return self._next_token

    @next_token.setter
    def next_token(self, value: str) -> None:
        self._next_token = value

    @next_token.deleter
    def next_token(self) -> None:
        self._next_token = UNSET



from typing import Any, cast, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..extensions import NotPresentError
from ..models.deprecated_entity_schema import DeprecatedEntitySchema
from ..types import UNSET, Unset

T = TypeVar("T", bound="DeprecatedEntitySchemasList")


@attr.s(auto_attribs=True, repr=False)
class DeprecatedEntitySchemasList:
    """  """

    _entity_schemas: Union[Unset, List[DeprecatedEntitySchema]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def __repr__(self):
        fields = []
        fields.append("entity_schemas={}".format(repr(self._entity_schemas)))
        fields.append("additional_properties={}".format(repr(self.additional_properties)))
        return "DeprecatedEntitySchemasList({})".format(", ".join(fields))

    def to_dict(self) -> Dict[str, Any]:
        entity_schemas: Union[Unset, List[Any]] = UNSET
        if not isinstance(self._entity_schemas, Unset):
            entity_schemas = []
            for entity_schemas_item_data in self._entity_schemas:
                entity_schemas_item = entity_schemas_item_data.to_dict()

                entity_schemas.append(entity_schemas_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        # Allow the model to serialize even if it was created outside of the constructor, circumventing validation
        if entity_schemas is not UNSET:
            field_dict["entitySchemas"] = entity_schemas

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any], strict: bool = False) -> T:
        d = src_dict.copy()

        def get_entity_schemas() -> Union[Unset, List[DeprecatedEntitySchema]]:
            entity_schemas = []
            _entity_schemas = d.pop("entitySchemas")
            for entity_schemas_item_data in _entity_schemas or []:
                entity_schemas_item = DeprecatedEntitySchema.from_dict(entity_schemas_item_data, strict=False)

                entity_schemas.append(entity_schemas_item)

            return entity_schemas

        try:
            entity_schemas = get_entity_schemas()
        except KeyError:
            if strict:
                raise
            entity_schemas = cast(Union[Unset, List[DeprecatedEntitySchema]], UNSET)

        deprecated_entity_schemas_list = cls(
            entity_schemas=entity_schemas,
        )

        deprecated_entity_schemas_list.additional_properties = d
        return deprecated_entity_schemas_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties

    def get(self, key, default=None) -> Optional[Any]:
        return self.additional_properties.get(key, default)

    @property
    def entity_schemas(self) -> List[DeprecatedEntitySchema]:
        if isinstance(self._entity_schemas, Unset):
            raise NotPresentError(self, "entity_schemas")
        return self._entity_schemas

    @entity_schemas.setter
    def entity_schemas(self, value: List[DeprecatedEntitySchema]) -> None:
        self._entity_schemas = value

    @entity_schemas.deleter
    def entity_schemas(self) -> None:
        self._entity_schemas = UNSET



import datetime
from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.bad_request_error import BadRequestError
from ...types import Response, UNSET, Unset


def _get_kwargs(
    *,
    client: Client,
    ids: Union[Unset, str] = UNSET,
    schema_id: Union[Unset, str] = UNSET,
    node_config_id: Union[Unset, str] = UNSET,
    flowchart_task_group_id: Union[Unset, str] = UNSET,
    folder_id: Union[Unset, str] = UNSET,
    project_id: Union[Unset, str] = UNSET,
    mentioned_in: Union[Unset, str] = UNSET,
    watcher_ids: Union[Unset, str] = UNSET,
    execution_types: Union[Unset, str] = UNSET,
    responsible_team_ids: Union[Unset, str] = UNSET,
    flowchart_workflow_task_group_ids: Union[Unset, str] = UNSET,
    flowchart_task_schema_id: Union[Unset, str] = UNSET,
    flowchart_creator_ids: Union[Unset, str] = UNSET,
    flowchart_watcher_ids: Union[Unset, str] = UNSET,
    status_idsany_of: Union[Unset, str] = UNSET,
    status_idsnone_of: Union[Unset, str] = UNSET,
    status_idsonly: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    name_includes: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
    modified_at: Union[Unset, datetime.date] = UNSET,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    display_ids: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/workflow-task-groups".format(client.base_url)

    headers: Dict[str, Any] = client.httpx_client.headers
    headers.update(client.get_headers())

    cookies: Dict[str, Any] = client.httpx_client.cookies
    cookies.update(client.get_cookies())

    json_modified_at: Union[Unset, str] = UNSET
    if not isinstance(modified_at, Unset):
        json_modified_at = modified_at.isoformat()

    params: Dict[str, Any] = {}
    if not isinstance(ids, Unset) and ids is not None:
        params["ids"] = ids
    if not isinstance(schema_id, Unset) and schema_id is not None:
        params["schemaId"] = schema_id
    if not isinstance(node_config_id, Unset) and node_config_id is not None:
        params["nodeConfigId"] = node_config_id
    if not isinstance(flowchart_task_group_id, Unset) and flowchart_task_group_id is not None:
        params["flowchartTaskGroupId"] = flowchart_task_group_id
    if not isinstance(folder_id, Unset) and folder_id is not None:
        params["folderId"] = folder_id
    if not isinstance(project_id, Unset) and project_id is not None:
        params["projectId"] = project_id
    if not isinstance(mentioned_in, Unset) and mentioned_in is not None:
        params["mentionedIn"] = mentioned_in
    if not isinstance(watcher_ids, Unset) and watcher_ids is not None:
        params["watcherIds"] = watcher_ids
    if not isinstance(execution_types, Unset) and execution_types is not None:
        params["executionTypes"] = execution_types
    if not isinstance(responsible_team_ids, Unset) and responsible_team_ids is not None:
        params["responsibleTeamIds"] = responsible_team_ids
    if (
        not isinstance(flowchart_workflow_task_group_ids, Unset)
        and flowchart_workflow_task_group_ids is not None
    ):
        params["flowchartWorkflowTaskGroupIds"] = flowchart_workflow_task_group_ids
    if not isinstance(flowchart_task_schema_id, Unset) and flowchart_task_schema_id is not None:
        params["flowchartTaskSchemaId"] = flowchart_task_schema_id
    if not isinstance(flowchart_creator_ids, Unset) and flowchart_creator_ids is not None:
        params["flowchartCreatorIds"] = flowchart_creator_ids
    if not isinstance(flowchart_watcher_ids, Unset) and flowchart_watcher_ids is not None:
        params["flowchartWatcherIds"] = flowchart_watcher_ids
    if not isinstance(status_idsany_of, Unset) and status_idsany_of is not None:
        params["statusIds.anyOf"] = status_idsany_of
    if not isinstance(status_idsnone_of, Unset) and status_idsnone_of is not None:
        params["statusIds.noneOf"] = status_idsnone_of
    if not isinstance(status_idsonly, Unset) and status_idsonly is not None:
        params["statusIds.only"] = status_idsonly
    if not isinstance(name, Unset) and name is not None:
        params["name"] = name
    if not isinstance(name_includes, Unset) and name_includes is not None:
        params["nameIncludes"] = name_includes
    if not isinstance(creator_ids, Unset) and creator_ids is not None:
        params["creatorIds"] = creator_ids
    if not isinstance(json_modified_at, Unset) and json_modified_at is not None:
        params["modifiedAt"] = json_modified_at
    if not isinstance(next_token, Unset) and next_token is not None:
        params["nextToken"] = next_token
    if not isinstance(page_size, Unset) and page_size is not None:
        params["pageSize"] = page_size
    if not isinstance(display_ids, Unset) and display_ids is not None:
        params["displayIds"] = display_ids
    if not isinstance(archive_reason, Unset) and archive_reason is not None:
        params["archiveReason"] = archive_reason

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[BadRequestError]:
    if response.status_code == 400:
        response_400 = BadRequestError.from_dict(response.json(), strict=False)

        return response_400
    return None


def _build_response(*, response: httpx.Response) -> Response[BadRequestError]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    ids: Union[Unset, str] = UNSET,
    schema_id: Union[Unset, str] = UNSET,
    node_config_id: Union[Unset, str] = UNSET,
    flowchart_task_group_id: Union[Unset, str] = UNSET,
    folder_id: Union[Unset, str] = UNSET,
    project_id: Union[Unset, str] = UNSET,
    mentioned_in: Union[Unset, str] = UNSET,
    watcher_ids: Union[Unset, str] = UNSET,
    execution_types: Union[Unset, str] = UNSET,
    responsible_team_ids: Union[Unset, str] = UNSET,
    flowchart_workflow_task_group_ids: Union[Unset, str] = UNSET,
    flowchart_task_schema_id: Union[Unset, str] = UNSET,
    flowchart_creator_ids: Union[Unset, str] = UNSET,
    flowchart_watcher_ids: Union[Unset, str] = UNSET,
    status_idsany_of: Union[Unset, str] = UNSET,
    status_idsnone_of: Union[Unset, str] = UNSET,
    status_idsonly: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    name_includes: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
    modified_at: Union[Unset, datetime.date] = UNSET,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    display_ids: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
) -> Response[BadRequestError]:
    kwargs = _get_kwargs(
        client=client,
        ids=ids,
        schema_id=schema_id,
        node_config_id=node_config_id,
        flowchart_task_group_id=flowchart_task_group_id,
        folder_id=folder_id,
        project_id=project_id,
        mentioned_in=mentioned_in,
        watcher_ids=watcher_ids,
        execution_types=execution_types,
        responsible_team_ids=responsible_team_ids,
        flowchart_workflow_task_group_ids=flowchart_workflow_task_group_ids,
        flowchart_task_schema_id=flowchart_task_schema_id,
        flowchart_creator_ids=flowchart_creator_ids,
        flowchart_watcher_ids=flowchart_watcher_ids,
        status_idsany_of=status_idsany_of,
        status_idsnone_of=status_idsnone_of,
        status_idsonly=status_idsonly,
        name=name,
        name_includes=name_includes,
        creator_ids=creator_ids,
        modified_at=modified_at,
        next_token=next_token,
        page_size=page_size,
        display_ids=display_ids,
        archive_reason=archive_reason,
    )

    response = client.httpx_client.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    ids: Union[Unset, str] = UNSET,
    schema_id: Union[Unset, str] = UNSET,
    node_config_id: Union[Unset, str] = UNSET,
    flowchart_task_group_id: Union[Unset, str] = UNSET,
    folder_id: Union[Unset, str] = UNSET,
    project_id: Union[Unset, str] = UNSET,
    mentioned_in: Union[Unset, str] = UNSET,
    watcher_ids: Union[Unset, str] = UNSET,
    execution_types: Union[Unset, str] = UNSET,
    responsible_team_ids: Union[Unset, str] = UNSET,
    flowchart_workflow_task_group_ids: Union[Unset, str] = UNSET,
    flowchart_task_schema_id: Union[Unset, str] = UNSET,
    flowchart_creator_ids: Union[Unset, str] = UNSET,
    flowchart_watcher_ids: Union[Unset, str] = UNSET,
    status_idsany_of: Union[Unset, str] = UNSET,
    status_idsnone_of: Union[Unset, str] = UNSET,
    status_idsonly: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    name_includes: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
    modified_at: Union[Unset, datetime.date] = UNSET,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    display_ids: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
) -> Optional[BadRequestError]:
    """ List workflow task groups """

    return sync_detailed(
        client=client,
        ids=ids,
        schema_id=schema_id,
        node_config_id=node_config_id,
        flowchart_task_group_id=flowchart_task_group_id,
        folder_id=folder_id,
        project_id=project_id,
        mentioned_in=mentioned_in,
        watcher_ids=watcher_ids,
        execution_types=execution_types,
        responsible_team_ids=responsible_team_ids,
        flowchart_workflow_task_group_ids=flowchart_workflow_task_group_ids,
        flowchart_task_schema_id=flowchart_task_schema_id,
        flowchart_creator_ids=flowchart_creator_ids,
        flowchart_watcher_ids=flowchart_watcher_ids,
        status_idsany_of=status_idsany_of,
        status_idsnone_of=status_idsnone_of,
        status_idsonly=status_idsonly,
        name=name,
        name_includes=name_includes,
        creator_ids=creator_ids,
        modified_at=modified_at,
        next_token=next_token,
        page_size=page_size,
        display_ids=display_ids,
        archive_reason=archive_reason,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    ids: Union[Unset, str] = UNSET,
    schema_id: Union[Unset, str] = UNSET,
    node_config_id: Union[Unset, str] = UNSET,
    flowchart_task_group_id: Union[Unset, str] = UNSET,
    folder_id: Union[Unset, str] = UNSET,
    project_id: Union[Unset, str] = UNSET,
    mentioned_in: Union[Unset, str] = UNSET,
    watcher_ids: Union[Unset, str] = UNSET,
    execution_types: Union[Unset, str] = UNSET,
    responsible_team_ids: Union[Unset, str] = UNSET,
    flowchart_workflow_task_group_ids: Union[Unset, str] = UNSET,
    flowchart_task_schema_id: Union[Unset, str] = UNSET,
    flowchart_creator_ids: Union[Unset, str] = UNSET,
    flowchart_watcher_ids: Union[Unset, str] = UNSET,
    status_idsany_of: Union[Unset, str] = UNSET,
    status_idsnone_of: Union[Unset, str] = UNSET,
    status_idsonly: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    name_includes: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
    modified_at: Union[Unset, datetime.date] = UNSET,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    display_ids: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
) -> Response[BadRequestError]:
    kwargs = _get_kwargs(
        client=client,
        ids=ids,
        schema_id=schema_id,
        node_config_id=node_config_id,
        flowchart_task_group_id=flowchart_task_group_id,
        folder_id=folder_id,
        project_id=project_id,
        mentioned_in=mentioned_in,
        watcher_ids=watcher_ids,
        execution_types=execution_types,
        responsible_team_ids=responsible_team_ids,
        flowchart_workflow_task_group_ids=flowchart_workflow_task_group_ids,
        flowchart_task_schema_id=flowchart_task_schema_id,
        flowchart_creator_ids=flowchart_creator_ids,
        flowchart_watcher_ids=flowchart_watcher_ids,
        status_idsany_of=status_idsany_of,
        status_idsnone_of=status_idsnone_of,
        status_idsonly=status_idsonly,
        name=name,
        name_includes=name_includes,
        creator_ids=creator_ids,
        modified_at=modified_at,
        next_token=next_token,
        page_size=page_size,
        display_ids=display_ids,
        archive_reason=archive_reason,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    ids: Union[Unset, str] = UNSET,
    schema_id: Union[Unset, str] = UNSET,
    node_config_id: Union[Unset, str] = UNSET,
    flowchart_task_group_id: Union[Unset, str] = UNSET,
    folder_id: Union[Unset, str] = UNSET,
    project_id: Union[Unset, str] = UNSET,
    mentioned_in: Union[Unset, str] = UNSET,
    watcher_ids: Union[Unset, str] = UNSET,
    execution_types: Union[Unset, str] = UNSET,
    responsible_team_ids: Union[Unset, str] = UNSET,
    flowchart_workflow_task_group_ids: Union[Unset, str] = UNSET,
    flowchart_task_schema_id: Union[Unset, str] = UNSET,
    flowchart_creator_ids: Union[Unset, str] = UNSET,
    flowchart_watcher_ids: Union[Unset, str] = UNSET,
    status_idsany_of: Union[Unset, str] = UNSET,
    status_idsnone_of: Union[Unset, str] = UNSET,
    status_idsonly: Union[Unset, str] = UNSET,
    name: Union[Unset, str] = UNSET,
    name_includes: Union[Unset, str] = UNSET,
    creator_ids: Union[Unset, str] = UNSET,
    modified_at: Union[Unset, datetime.date] = UNSET,
    next_token: Union[Unset, str] = UNSET,
    page_size: Union[Unset, int] = 50,
    display_ids: Union[Unset, str] = UNSET,
    archive_reason: Union[Unset, str] = UNSET,
) -> Optional[BadRequestError]:
    """ List workflow task groups """

    return (
        await asyncio_detailed(
            client=client,
            ids=ids,
            schema_id=schema_id,
            node_config_id=node_config_id,
            flowchart_task_group_id=flowchart_task_group_id,
            folder_id=folder_id,
            project_id=project_id,
            mentioned_in=mentioned_in,
            watcher_ids=watcher_ids,
            execution_types=execution_types,
            responsible_team_ids=responsible_team_ids,
            flowchart_workflow_task_group_ids=flowchart_workflow_task_group_ids,
            flowchart_task_schema_id=flowchart_task_schema_id,
            flowchart_creator_ids=flowchart_creator_ids,
            flowchart_watcher_ids=flowchart_watcher_ids,
            status_idsany_of=status_idsany_of,
            status_idsnone_of=status_idsnone_of,
            status_idsonly=status_idsonly,
            name=name,
            name_includes=name_includes,
            creator_ids=creator_ids,
            modified_at=modified_at,
            next_token=next_token,
            page_size=page_size,
            display_ids=display_ids,
            archive_reason=archive_reason,
        )
    ).parsed



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.async_task_link import AsyncTaskLink
from ...models.bad_request_error import BadRequestError
from ...models.multiple_containers_transfers_list import MultipleContainersTransfersList
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
    json_body: MultipleContainersTransfersList,
) -> Dict[str, Any]:
    url = "{}/transfers".format(client.base_url)

    headers: Dict[str, Any] = client.httpx_client.headers
    headers.update(client.get_headers())

    cookies: Dict[str, Any] = client.httpx_client.cookies
    cookies.update(client.get_cookies())

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[AsyncTaskLink, BadRequestError]]:
    if response.status_code == 202:
        response_202 = AsyncTaskLink.from_dict(response.json(), strict=False)

        return response_202
    if response.status_code == 400:
        response_400 = BadRequestError.from_dict(response.json(), strict=False)

        return response_400
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[AsyncTaskLink, BadRequestError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    json_body: MultipleContainersTransfersList,
) -> Response[Union[AsyncTaskLink, BadRequestError]]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    response = client.httpx_client.post(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    json_body: MultipleContainersTransfersList,
) -> Optional[Union[AsyncTaskLink, BadRequestError]]:
    """Transfers a volume of an entity, batch, or container into a destination container. Limit of 5000 transfers per request. Concentration of all contents in the destination container will be automatically updated based on the previous volume & concentrations of the contents in that container, the concentration of the contents being transferred in, the volume of the contents being transferred in, and the final volume of the container. If no concentration is specified, the concentration will not be tracked."""

    return sync_detailed(
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    json_body: MultipleContainersTransfersList,
) -> Response[Union[AsyncTaskLink, BadRequestError]]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    json_body: MultipleContainersTransfersList,
) -> Optional[Union[AsyncTaskLink, BadRequestError]]:
    """Transfers a volume of an entity, batch, or container into a destination container. Limit of 5000 transfers per request. Concentration of all contents in the destination container will be automatically updated based on the previous volume & concentrations of the contents in that container, the concentration of the contents being transferred in, the volume of the contents being transferred in, and the final volume of the container. If no concentration is specified, the concentration will not be tracked."""

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
        )
    ).parsed
